#ifndef _ANSWER_H_
#define _ANSWER_H_  1

#include <algorithm>
#include <iostream>
#include "csmmm.h"

#if BLACK_ONLY == 0 // white and black paegs in answers

// The number of possible answers
static const int possible_answers = (pegs * (pegs + 3) / 2);

static char const * const game_name = "f_";

class Answer {
  public:
    Answer() {}
    Answer(int w, int b) : marks(w + b), blacks(b) {}

    // Answer is coded as an integer, e.g.
    //           |     #marks
    //           |  0  1  2  3  4
    // ----------+---------------
    // #blacks 0 |  0  1  3  6 10
    //         1 |     2  4  7 11
    //         2 |        5  8 12
    //         3 |           9
    //         4 |             13
    // #marks = #blacks + #whites
    Answer(int code) {
      if (code == possible_answers - 1)
        marks = blacks = pegs;
      else {
        marks = 0;
        while (code > marks) {
          ++marks;
          code -= marks;
        }
        blacks = code;
      }
    }

    Answer(Configuration const &conf, Configuration const &question) {
      // Static tables are faster.
      static int color_conf[colors], color_question[colors];
      for (int i = 0; i < colors; ++i)
        color_conf[i] = color_question[i] = 0;

      blacks = 0;
      for (int i = 0; i < pegs; ++i) {
        if (conf[i] == question[i]) {
          ++blacks;
        }
        ++color_conf[(int)conf[i]];
        ++color_question[(int)question[i]];
      }

      marks = 0;
      for (int i = 0; i < colors; ++i) {
        marks += ::std::min(color_conf[i], color_question[i]);
      }
    }

    // Answer is coded as an integer, e.g.
    //           |     #whites
    //           |  0  1  2  3  4
    // ----------+---------------
    // #blacks 0 |  0  1  3  6 10
    //         1 |  2  4  7 11
    //         2 |  5  8 12
    //         3 |  9
    //         4 | 13
    // #marks = #blacks + #whites
    int Code() const {
      // int m = ((marks * (marks + 1)) >> 1) + blacks;
      int m = ((marks * marks + marks) >> 1) + blacks;
      if (m == possible_answers)
        --m;
      return m;
    }

    int B() const {
      return blacks;
    }

    int W() const {
      return marks - blacks;
    }

  private:
    int marks;  // The number of blacks + whites
    int blacks; // The number of blacks
};

static std::ostream & operator<<(std::ostream &os, Answer const &ans) {
  os << ans.W() << 'W' << ans.B() << 'B';
  return os;
}

#else // black-peg only answers

// The number of possible answers
static const int possible_answers = pegs + 1;

static char const * const game_name = "b_";

class Answer {
  public:
    Answer() {}
    Answer(int w, int b) : blacks(b) {}

    Answer(int code) {
      blacks = code;
    }

    Answer(Configuration const &conf, Configuration const &question) {
      blacks = 0;
      for (int i = 0; i < pegs; ++i) {
        if (conf[i] == question[i]) {
          ++blacks;
        }
      }
    }

    int Code() const {
      return blacks;
    }

    int B() const {
      return blacks;
    }

    int W() const {
      return 0;
    }

  private:
    int blacks; // The number of blacks
};

static std::ostream & operator<<(std::ostream &os, Answer const &ans) {
  os << ans.B() << 'B';
  return os;
}

#endif

#endif
